=begin
#ApproveAPISwagger

#The simple API to request a user's approval on anything via email + sms.

OpenAPI spec version: 1.0.1
Contact: dev@approveapi.com
Generated by: https://openapi-generator.tech
OpenAPI Generator version: 4.0.0-SNAPSHOT

=end

require 'date'

module ApproveAPISwagger
  class CreatePromptRequest
    # The user to send the approval request to. Can be either an email address or a phone number.
    attr_accessor :user

    # The body of the approval request to show the user.
    attr_accessor :body

    # The title of an approval request. Defaults to an empty string.
    attr_accessor :title

    # The approve action text. Defaults to 'Approve'.
    attr_accessor :approve_text

    # An HTTPS URL to redirect the user to if the prompt is approved. This URL is kept secret until the user is redirected to it.
    attr_accessor :approve_redirect_url

    # The reject action text. If not specified the reject button will NOT be rendered, and the user will only see an approve action button.
    attr_accessor :reject_text

    # An HTTPS URL to redirect the user to if the prompt is rejected. This URL is kept secret until the user is redirected to it.
    attr_accessor :reject_redirect_url

    # If true, the request waits (long-polls) until the user responds to the prompt or more than 10 minutes pass. Defaults to false.
    attr_accessor :long_poll

    # The number of seconds until this request can no longer be answered.
    attr_accessor :expires_in

    attr_accessor :metadata

    attr_accessor :internal_data

    # Allows calling `create_prompt` multiple times idempotently, such that a prompt is sent at-most once. This key should contain sufficient randomness. Idempotent requests are stored for 24 hours. After that time, the same key will create a new request.
    attr_accessor :idempotency_key

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'user' => :'user',
        :'body' => :'body',
        :'title' => :'title',
        :'approve_text' => :'approve_text',
        :'approve_redirect_url' => :'approve_redirect_url',
        :'reject_text' => :'reject_text',
        :'reject_redirect_url' => :'reject_redirect_url',
        :'long_poll' => :'long_poll',
        :'expires_in' => :'expires_in',
        :'metadata' => :'metadata',
        :'internal_data' => :'internal_data',
        :'idempotency_key' => :'idempotency_key'
      }
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'user' => :'String',
        :'body' => :'String',
        :'title' => :'String',
        :'approve_text' => :'String',
        :'approve_redirect_url' => :'String',
        :'reject_text' => :'String',
        :'reject_redirect_url' => :'String',
        :'long_poll' => :'BOOLEAN',
        :'expires_in' => :'Float',
        :'metadata' => :'PromptMetadata',
        :'internal_data' => :'Hash<String, String>',
        :'idempotency_key' => :'String'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'user')
        self.user = attributes[:'user']
      end

      if attributes.has_key?(:'body')
        self.body = attributes[:'body']
      end

      if attributes.has_key?(:'title')
        self.title = attributes[:'title']
      end

      if attributes.has_key?(:'approve_text')
        self.approve_text = attributes[:'approve_text']
      end

      if attributes.has_key?(:'approve_redirect_url')
        self.approve_redirect_url = attributes[:'approve_redirect_url']
      end

      if attributes.has_key?(:'reject_text')
        self.reject_text = attributes[:'reject_text']
      end

      if attributes.has_key?(:'reject_redirect_url')
        self.reject_redirect_url = attributes[:'reject_redirect_url']
      end

      if attributes.has_key?(:'long_poll')
        self.long_poll = attributes[:'long_poll']
      end

      if attributes.has_key?(:'expires_in')
        self.expires_in = attributes[:'expires_in']
      end

      if attributes.has_key?(:'metadata')
        self.metadata = attributes[:'metadata']
      end

      if attributes.has_key?(:'internal_data')
        if (value = attributes[:'internal_data']).is_a?(Hash)
          self.internal_data = value
        end
      end

      if attributes.has_key?(:'idempotency_key')
        self.idempotency_key = attributes[:'idempotency_key']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if @user.nil?
        invalid_properties.push('invalid value for "user", user cannot be nil.')
      end

      if @body.nil?
        invalid_properties.push('invalid value for "body", body cannot be nil.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if @user.nil?
      return false if @body.nil?
      true
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          user == o.user &&
          body == o.body &&
          title == o.title &&
          approve_text == o.approve_text &&
          approve_redirect_url == o.approve_redirect_url &&
          reject_text == o.reject_text &&
          reject_redirect_url == o.reject_redirect_url &&
          long_poll == o.long_poll &&
          expires_in == o.expires_in &&
          metadata == o.metadata &&
          internal_data == o.internal_data &&
          idempotency_key == o.idempotency_key
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [user, body, title, approve_text, approve_redirect_url, reject_text, reject_redirect_url, long_poll, expires_in, metadata, internal_data, idempotency_key].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.openapi_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        ApproveAPISwagger.const_get(type).build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
